'' =================================================================================================
''
''   File....... jm_iso2_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 20 JAN 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  CLD2 =  4                                                     ' card at end stop
  CLD1 =  3                                                     ' card detected
  CLD  =  2                                                     ' card in motion
  RXL  =  1                                                     ' clock
  RDT  =  0                                                     ' data

  
con 
  
  #(-1), IS_ON, IS_OFF
  
  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"
  iso2 : "jm_iso2_reader"
  str  : "jm_strings"
 

var


pub main | check, card

  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
  iso2.start(RDT)                                               ' start iso2 reader

  pause(10)

  term.tx(CLS)
  term.str(string("ISO2 Card Reader", CR, CR))

  repeat
    term.str(string("Waiting...", CR))                          ' indicate ready

    repeat until iso2.card_inserted                             ' hold for card
    check := \iso2.read_card(1000)                              ' read the card, 1s time-out

    if (check => 0)                                             ' if good read
      {
      term.str(@Arrow)        
      term.str(check)   
      term.tx(CR)
      }             
    
      card := card_type                                         ' determine card type
      if (check_luhn(iso2.bufstr) == false)                     ' if bogus
        card := 0                                               '  remove type
      term.str(@Arrow)
      term.str(iso2.bufstr)                                     ' show card data
      term.tx(CR)
      term.str(@Arrow)
      term.str(@@CTString[card])                                ' show type string  
      term.tx(CR) 
    else
      term.str(@Arrow)
      term.str(iso2.errstr(check))                              ' show reason for abort
      term.tx(CR)      

    repeat while iso2.card_inserted                             ' wait until card is removed
    term.tx(CR)


pub card_type | ccstr, ch, check

'' Attempt to determine card type
'' -- reduces buffer to 1st field
'' -- based on information from http://stevemorse.org/ssn/cc.html

  iso2.clear_field_sep                                          ' reduce to single field

  ccstr := iso2.bufstr                                          ' get string address
  ch    := byte[ccstr]                                          ' get leading character

  case ch
    "0", "7".."9":
      return 0                                                  ' unknown

    "1":
      check := str.asc2dec(ccstr, 4)                            ' convert 1st 4 digits to number
      case check
        1800:
          return 1                                              ' jcb
        other:
          return 0                                              ' unknown

    "2":
      check := str.asc2dec(ccstr, 4)
      case check
        2014, 2149:
          return 2                                              ' diner's clube / enroute     
        2131:
          return 1                                              ' jcb     
        other:
          return 0

    "3":
      check := str.asc2dec(ccstr, 3)
      case check
        300..305, 380..389:
          return 3                                              ' diner's club / carte blanche   
        360..369:
          return 4                                              ' diner's club / international
        340..349, 370..379:
          return 5                                              ' amex    
        other:
          return 1                                              ' jcb 

    "4":
      return 6                                                  ' visa

    "5":
      check := str.asc2dec(ccstr, 2)
      case check 
        51..55:
          return 7                                              ' mastercard       
        other:
          return 0                                              ' unknown   

    "6":
      check := str.asc2dec(ccstr, 4)
      case check
        6011:
          return 8                                              ' discover
        other:
          return 0                                              ' unknown

        
pub check_luhn(p_str) | sum, ndigits, parity, idx, n

'' Validates credit card # (at p_str) using the Luhn check algorithm
'' -- http://www.codeproject.com/Articles/10720/How-To-Validate-Credit-Card-Numbers

  sum     := 0                                                  ' clear Luhn sum
  ndigits := strsize(p_str)                                     ' get # of digits in CC #
  parity  := ndigits & 1                                        ' calc parity position

  repeat idx from 0 to ndigits-1                                ' loop through string
    n := byte[p_str+idx] - "0"                                  ' convert char to digit
    if ((idx & 1) == parity)                                    ' parity position?
      n <<= 1                                                   '  double it
    if (n > 9)                                                  ' if > 9
      n -= 9                                                    '  reduce to single digit
    sum += n                                                    ' update sum
    
  return ((sum // 10) == 0)                                     ' return true if divisible by 10


dat

Arrow           byte    "--> ", 0


'' Credit card types

CType00         byte    "Unknown", 0
CType01         byte    "JCB", 0
CType02         byte    "Diner's Club / enRoute", 0
CType03         byte    "Diner's Club / Carte Blanche", 0
CType04         byte    "Diner's Club / International", 0 
CType05         byte    "American Express", 0
CType06         byte    "VISA", 0
CType07         byte    "MasterCard", 0
CType08         byte    "Discover", 0           

CTString        word    @CType00, @CType01, @CType02
                word    @CType03, @CType04, @CType05
                word    @CType06, @CType07, @CType08  


  
con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}